@@ -1,7 +1,5 @@
 import base64
-from tests.test_cli import test_query_memory_does_not_create_file
 import click
-import codecs
 from click_default_group import DefaultGroup
 from datetime import datetime
 import hashlib
@@ -9,6 +7,7 @@ import pathlib
 import sqlite_utils
 from sqlite_utils.db import AlterError
 import textwrap
+import io
 import itertools
 import json
 import os
@@ -32,6 +31,18 @@ It's often worth trying: --encoding=latin-1
 """.strip()
 
 
+# Increase CSV field size limit to maximim possible
+# https://stackoverflow.com/a/15063941
+field_size_limit = sys.maxsize
+
+while True:
+    try:
+        csv_std.field_size_limit(field_size_limit)
+        break
+    except OverflowError:
+        field_size_limit = int(field_size_limit / 10)
+
+
 def output_options(fn):
     for decorator in reversed(
         (
@@ -604,6 +615,12 @@ def insert_upsert_options(fn):
             click.option("--tsv", is_flag=True, help="Expect TSV"),
             click.option("--delimiter", help="Delimiter to use for CSV files"),
             click.option("--quotechar", help="Quote character to use for CSV/TSV"),
+            click.option(
+                "--sniff", is_flag=True, help="Detect delimiter and quote character"
+            ),
+            click.option(
+                "--no-headers", is_flag=True, help="CSV file has no header row"
+            ),
             click.option(
                 "--batch-size", type=int, default=100, help="Commit every X records"
             ),
@@ -645,6 +662,8 @@ def insert_upsert_implementation(
     tsv,
     delimiter,
     quotechar,
+    sniff,
+    no_headers,
     batch_size,
     alter,
     upsert,
@@ -659,33 +678,44 @@ def insert_upsert_implementation(
 ):
     db = sqlite_utils.Database(path)
     _load_extensions(db, load_extension)
-    if delimiter or quotechar:
+    if delimiter or quotechar or sniff or no_headers:
         csv = True
     if (nl + csv + tsv) >= 2:
         raise click.ClickException("Use just one of --nl, --csv or --tsv")
     if encoding and not (csv or tsv):
         raise click.ClickException("--encoding must be used with --csv or --tsv")
     encoding = encoding or "utf-8"
-    json_file = codecs.getreader(encoding)(json_file)
+    buffered = io.BufferedReader(json_file, buffer_size=4096)
+    decoded = io.TextIOWrapper(buffered, encoding=encoding)
     if pk and len(pk) == 1:
         pk = pk[0]
     if csv or tsv:
-        dialect = "excel-tab" if tsv else "excel"
-        with file_progress(json_file, silent=silent) as json_file:
+        if sniff:
+            # Read first 2048 bytes and use that to detect
+            first_bytes = buffered.peek(2048)
+            dialect = csv_std.Sniffer().sniff(first_bytes.decode(encoding, "ignore"))
+        else:
+            dialect = "excel-tab" if tsv else "excel"
+        with file_progress(decoded, silent=silent) as decoded:
             csv_reader_args = {"dialect": dialect}
             if delimiter:
                 csv_reader_args["delimiter"] = delimiter
             if quotechar:
                 csv_reader_args["quotechar"] = quotechar
-            reader = csv_std.reader(json_file, **csv_reader_args)
-            headers = next(reader)
+            reader = csv_std.reader(decoded, **csv_reader_args)
+            first_row = next(reader)
+            if no_headers:
+                headers = ["untitled_{}".format(i + 1) for i in range(len(first_row))]
+                reader = itertools.chain([first_row], reader)
+            else:
+                headers = first_row
             docs = (dict(zip(headers, row)) for row in reader)
     else:
         try:
             if nl:
-                docs = (json.loads(line) for line in json_file)
+                docs = (json.loads(line) for line in decoded)
             else:
-                docs = json.load(json_file)
+                docs = json.load(decoded)
                 if isinstance(docs, dict):
                     docs = [docs]
         except json.decoder.JSONDecodeError:
@@ -734,6 +764,8 @@ def insert(
     tsv,
     delimiter,
     quotechar,
+    sniff,
+    no_headers,
     batch_size,
     alter,
     encoding,
@@ -762,6 +794,8 @@ def insert(
             tsv,
             delimiter,
             quotechar,
+            sniff,
+            no_headers,
             batch_size,
             alter=alter,
             upsert=False,
@@ -791,6 +825,8 @@ def upsert(
     batch_size,
     delimiter,
     quotechar,
+    sniff,
+    no_headers,
     alter,
     not_null,
     default,
@@ -814,6 +850,8 @@ def upsert(
             tsv,
             delimiter,
             quotechar,
+            sniff,
+            no_headers,
             batch_size,
             alter=alter,
             upsert=True,
@@ -982,6 +1020,12 @@ def drop_view(path, view, load_extension):
     required=True,
 )
 @click.argument("sql")
+@click.option(
+    "--attach",
+    type=(str, click.Path(file_okay=True, dir_okay=False, allow_dash=False)),
+    multiple=True,
+    help="Additional databases to attach - specify alias and filepath",
+)
 @output_options
 @click.option("-r", "--raw", is_flag=True, help="Raw output, first column of first row")
 @click.option(
@@ -995,6 +1039,7 @@ def drop_view(path, view, load_extension):
 def query(
     path,
     sql,
+    attach,
     nl,
     arrays,
     csv,
@@ -1009,6 +1054,8 @@ def query(
 ):
     "Execute SQL query and return the results as JSON"
     db = sqlite_utils.Database(path)
+    for alias, attach_path in attach:
+        db.attach(alias, attach_path)
     _load_extensions(db, load_extension)
     db.register_fts4_bm25()
     with db.conn:
