@@ -219,6 +219,14 @@ class Database:
     def register_fts4_bm25(self):
         self.register_function(rank_bm25, deterministic=True)
 
+    def attach(self, alias, filepath):
+        attach_sql = """
+            ATTACH DATABASE '{}' AS [{}];
+        """.format(
+            str(pathlib.Path(filepath).resolve()), alias
+        ).strip()
+        self.execute(attach_sql)
+
     def execute(self, sql, parameters=None):
         if self._tracer:
             self._tracer(sql, parameters)
@@ -644,7 +652,15 @@ class Queryable:
     def rows(self):
         return self.rows_where()
 
-    def rows_where(self, where=None, where_args=None, order_by=None, select="*"):
+    def rows_where(
+        self,
+        where=None,
+        where_args=None,
+        order_by=None,
+        select="*",
+        limit=None,
+        offset=None,
+    ):
         if not self.exists():
             return []
         sql = "select {} from [{}]".format(select, self.name)
@@ -652,6 +668,10 @@ class Queryable:
             sql += " where " + where
         if order_by is not None:
             sql += " order by " + order_by
+        if limit is not None:
+            sql += " limit {}".format(limit)
+        if offset is not None:
+            sql += " offset {}".format(offset)
         cursor = self.db.execute(sql, where_args or [])
         columns = [c[0] for c in cursor.description]
         for row in cursor:
@@ -1454,7 +1474,7 @@ class Table(Queryable):
             )
         return self
 
-    def search_sql(self, columns=None, order_by=None, limit=None):
+    def search_sql(self, columns=None, order_by=None, limit=None, offset=None):
         # Pick names for table and rank column that don't clash
         original = "original_" if self.name == "original" else "original"
         columns_sql = "*"
@@ -1486,7 +1506,7 @@ class Table(Queryable):
             [{fts_table}] match :query
         order by
             {order_by}
-        {limit}
+        {limit_offset}
         """
         ).strip()
         if virtual_table_using == "FTS5":
@@ -1496,6 +1516,11 @@ class Table(Queryable):
             rank_implementation = "rank_bm25(matchinfo([{}], 'pcnalx'))".format(
                 fts_table
             )
+        limit_offset = ""
+        if limit is not None:
+            limit_offset += " limit {}".format(limit)
+        if offset is not None:
+            limit_offset += " offset {}".format(offset)
         return sql.format(
             dbtable=self.name,
             original=original,
@@ -1503,15 +1528,16 @@ class Table(Queryable):
             columns_with_prefix=columns_with_prefix_sql,
             fts_table=fts_table,
             order_by=order_by or rank_implementation,
-            limit="limit {}".format(limit) if limit else "",
+            limit_offset=limit_offset.strip(),
         ).strip()
 
-    def search(self, q, order_by=None, columns=None, limit=None):
+    def search(self, q, order_by=None, columns=None, limit=None, offset=None):
         cursor = self.db.execute(
             self.search_sql(
                 order_by=order_by,
                 columns=columns,
                 limit=limit,
+                offset=offset,
             ),
             {"query": q},
         )
@@ -1886,12 +1912,10 @@ class Table(Queryable):
                 if hash_id:
                     all_columns.insert(0, hash_id)
             else:
-                all_columns += [
-                    column
-                    for record in chunk
-                    for column in record
-                    if column not in all_columns
-                ]
+                for record in chunk:
+                    all_columns += [
+                        column for column in record if column not in all_columns
+                    ]
 
             validate_column_names(all_columns)
             first = False
@@ -1973,9 +1997,9 @@ class Table(Queryable):
 
     def add_missing_columns(self, records):
         needed_columns = suggest_column_types(records)
-        current_columns = self.columns_dict
+        current_columns = {c.lower() for c in self.columns_dict}
         for col_name, col_type in needed_columns.items():
-            if col_name not in current_columns:
+            if col_name.lower() not in current_columns:
                 self.add_column(col_name, col_type)
         return self
 
@@ -2009,6 +2033,7 @@ class Table(Queryable):
         pk=DEFAULT,
         lookup=None,
         m2m_table=None,
+        alter=False,
     ):
         if isinstance(other_table, str):
             other_table = self.db.table(other_table, pk=pk)
@@ -2045,7 +2070,9 @@ class Table(Queryable):
             )
             # Ensure each record exists in other table
             for record in records:
-                id = other_table.insert(record, pk=pk, replace=True).last_pk
+                id = other_table.insert(
+                    record, pk=pk, replace=True, alter=alter
+                ).last_pk
                 m2m_table.insert(
                     {
                         "{}_id".format(other_table.name): id,
@@ -2143,6 +2170,11 @@ class View(Queryable):
     def drop(self):
         self.db.execute("DROP VIEW [{}]".format(self.name))
 
+    def enable_fts(self, *args, **kwargs):
+        raise NotImplementedError(
+            "enable_fts() is supported on tables but not on views"
+        )
+
 
 def chunks(sequence, size):
     iterator = iter(sequence)
